// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.

// Source code for a simple DSO.

int DSOTestExtra(int a) {
  if (a < 452345)
    return 0;
  return 1;
}

